<?php
/**
 * Displays and acts on the plugin's options page.
 *
 * @since 0.3.0
 * @package ethpress
 */

namespace losnappas\Ethpress\Admin;

defined( 'ABSPATH' ) || die;

/**
 * Static.
 *
 * @since 0.3.0
 */
class Options {
	/**
	 * Adds options page.
	 *
	 * @since 0.3.0
	 */
	public static function admin_menu() {
		$page = esc_html__( 'EthPress', 'ethpress' );
		if ( is_multisite() ) {
			add_submenu_page(
				'settings.php',
				$page,
				$page,
				'manage_network_options',
				'ethpress',
				[ __CLASS__, 'create_page' ]
			);
		} else {
			add_options_page(
				$page,
				$page,
				'manage_options',
				'ethpress',
				[ __CLASS__, 'create_page' ]
			);
		}
	}

	/**
	 * Creates options page.
	 *
	 * @since 0.3.0
	 */
	public static function create_page() {
		?>
		<div class="wrap">
			<h1><?php esc_html_e( 'EthPress Options Page', 'ethpress' ); ?></h1>
			<p><?php esc_html_e( 'The MetaMask login plugin.', 'ethpress' ); ?></p>
			<?php if ( is_multisite() ) { ?>
				<form action="../options.php" method="POST">
			<?php } else { ?>
				<form action="options.php" method="POST">
				<?php
			}
			settings_fields( 'ethpress' );
			do_settings_sections( 'ethpress' );
			submit_button();
			?>
			</form>
			</div>
		<?php
	}

	/**
	 * Adds settings for api_url to options page. admin_init hooked.
	 *
	 * @since 0.3.0
	 */
	public static function admin_init() {
		register_setting(
			'ethpress',
			'ethpress',
			[ __CLASS__, 'options_validate' ]
		);
		add_settings_section(
			'ethpress_main',
			esc_html__( 'Main Settings', 'ethpress' ),
			[ __CLASS__, 'section_api_url' ],
			'ethpress'
		);
		add_settings_field(
			'ethpress_api_url',
			esc_html__( 'Verification service', 'ethpress' ),
			[ __CLASS__, 'input_api_url' ],
			'ethpress',
			'ethpress_main'
		);
	}

	/**
	 * Outputs section title.
	 *
	 * @since 0.3.0
	 */
	public static function section_api_url() {
	}

	/**
	 * Outputs input for api url option.
	 *
	 * @since 0.3.0
	 */
	public static function input_api_url() {
		$ecrecovers_with_php = extension_loaded( 'gmp' ) || extension_loaded( 'bcmath' );
		if ( $ecrecovers_with_php ) {
			echo '<p>' . esc_html__( 'Your PHP installation has the necessary PHP extension to do verifications on your server, so there is nothing to configure.', 'ethpress' ) . '</p>';
			// Open hiding div.
			echo '<div style="display: none;">';
		}
		$options = get_site_option( 'ethpress' );
		echo '<input class="regular-text" id="ethpress_api_url" name="ethpress[api_url]" type="text" value="' . esc_attr( esc_url( $options['api_url'] ) ) . '" />';
		echo '<p class="description">' . esc_html__( 'Use an API or install php-gmp or php-bcmath to verify Ethereum signatures.', 'ethpress' ) . '</p>';
		echo '<p class="description"><a href="https://www.php.net/manual/en/book.gmp.php">PHP-GMP</a> / <a href="https://www.php.net/manual/en/book.bc.php">PHP-BCMath</a></p>';
		echo '<p class="description">' . esc_html__( 'Default API url:', 'ethpress' ) . ' https://verify-eth-signature.herokuapp.com/ethpress</p>';
		echo '<p class="description">' . wp_kses(
			sprintf(
				/* translators: a link. */
				__( 'For more information, see %1$s.', 'ethpress' ),
				'<a href="https://gitlab.com/losnappas/verify-eth-signature" target="_blank" rel="noopener noreferrer">https://gitlab.com/losnappas/verify-eth-signature</a>'
			),
			[
				'a' => [
					'href'   => [],
					'target' => [],
					'rel'    => [],
				],
			]
		) . '</p>';
		if ( $ecrecovers_with_php ) {
			// Close hiding div.
			echo '</div>';
		}
	}

	/**
	 * Validates input for api url option.
	 *
	 * @param array $input New options input.
	 *
	 * @since 0.3.0
	 */
	public static function options_validate( $input ) {
		$options = get_site_option( 'ethpress' );
		$newurl  = esc_url_raw( trim( $input['api_url'] ) );
		if ( empty( $input['recursive'] ) && is_multisite() ) {
			$options['api_url'] = $newurl;
			// Mark next call as recursed.
			$options['recursive'] = true;
			// This calls this validation function recursively.
			update_site_option( 'ethpress', $options );
		}
		$options['api_url'] = $newurl;
		return $options;
	}
}
