<?php
/**
 * Sets up the plugin's hooks.
 *
 * @since 0.1.0
 * @package ethpress
 */

namespace losnappas\Ethpress;

defined( 'ABSPATH' ) || die;

use losnappas\Ethpress\Upgrade;
use losnappas\Ethpress\Shortcodes\LoginButton;

/**
 * Static functions only.
 *
 * @since 0.1.0
 */
class Plugin {

	/**
	 * Table names. Don't forget to base_prefix. Base_ because it's user related and does not have separate tables for every site.
	 *
	 * @since 0.1.0
	 *
	 * @var array
	 */
	public static $tables = [
		'addresses' => 'ethpress_addresses',
	];

	/**
	 * This contains hook and filter assignments, etc.
	 *
	 * @since 0.1.0
	 */
	public static function attach_hooks() {
		add_action( 'plugins_loaded', [ __CLASS__, 'load_plugin_textdomain' ] );

		// When bbPress does `do_action('login_form')`, scripts aren't loaded without these 2.
		// If bbPress does that, then others will too.
		// Not sure what's going on, but this is an easy fix.
		add_action( 'login_form', [ __CLASS__, 'register_scripts' ], 9 );
		add_action( 'login_form', [ __CLASS__, 'login_enqueue_scripts' ] );
		add_action( 'login_form', [ ETHPRESS_NS . '\Front', 'login_form' ] );
		/**
		 * Adding register_scripts to wp_enqueue_scripts for shortcode support.
		 * I think the performance impact of this is next to nothing.
		 *
		 * @since 0.5.0
		 */
		add_action( 'wp_enqueue_scripts', [ __CLASS__, 'register_scripts' ], 9 );
		add_action( 'login_enqueue_scripts', [ __CLASS__, 'register_scripts' ], 9 );
		// By default we only enqueue for login screen.
		add_action( 'login_enqueue_scripts', [ __CLASS__, 'login_enqueue_scripts' ] );

		add_action( 'wp_ajax_nopriv_ethpress_log_in', [ ETHPRESS_NS . '\Login', 'verify_login' ] );
		add_action( 'wp_ajax_nopriv_ethpress_get_message', [ ETHPRESS_NS . '\Login', 'get_message' ] );
		add_action( 'wp_ajax_ethpress_log_in', [ ETHPRESS_NS . '\Login', 'verify_login' ] );
		add_action( 'wp_ajax_ethpress_get_message', [ ETHPRESS_NS . '\Login', 'get_message' ] );
		add_action( 'deleted_user', [ ETHPRESS_NS . '\Login', 'destroy' ] );

		add_action( 'ethpress_login', [ ETHPRESS_NS . '\Login', 'attach_user_to_blog' ] );

		add_shortcode( LoginButton::$shortcode_name, [ ETHPRESS_NS . '\Shortcodes\LoginButton', 'add_shortcode' ] );

		self::attach_admin_hooks();
	}

	/**
	 * Attaches admin hooks for options page.
	 *
	 * @since 0.3.0
	 */
	public static function attach_admin_hooks() {
		if ( is_multisite() ) {
			add_action( 'network_admin_menu', [ ETHPRESS_NS . '\Admin\Options', 'admin_menu' ] );
		} else {
			add_action( 'admin_menu', [ ETHPRESS_NS . '\Admin\Options', 'admin_menu' ] );
		}
		add_action( 'admin_init', [ ETHPRESS_NS . '\Admin\Options', 'admin_init' ] );
	}

	/**
	 * Registers the scripts so they're available in login_enqueue_scripts and wp_enqueue_scripts.
	 *
	 * @since 0.1.1
	 */
	public static function register_scripts() {
		if ( wp_script_is( 'wp-login-modal', 'registered' ) ) {
			return;
		}
		// Styles for just the button.
		wp_register_style(
			'ethpress-login',
			plugin_dir_url( ETHPRESS_FILE ) . 'public/css/login.css',
			[],
			'1.0.0'
		);
		/**
		 * Version 0.5.0 removed the info popup and thus modified html. This fixes css while keeping 'ethpress-login-front' script functional.
		 */
		wp_register_style(
			'ethpress-login-front',
			plugin_dir_url( ETHPRESS_FILE ) . 'public/css/login-fix.css',
			[ 'ethpress-login' ],
			'1.0.0'
		);
		// Web3Login.js
		// Provides ethpress.metamask.connect. Loaded in header for easier access.
		wp_register_script(
			'ethpress-login-modal',
			plugin_dir_url( ETHPRESS_FILE ) . 'public/dist/Web3Login/index.min.js',
			[],
			'1.3.0',
			false
		);
		// Configures Web3Login to work.
		wp_register_script(
			'ethpress-login',
			plugin_dir_url( ETHPRESS_FILE ) . 'public/dist/main.min.js',
			[ 'ethpress-login-modal' ],
			'1.3.0',
			false
		);
		// Adds event listener to buttons.
		wp_register_script(
			'ethpress-login-front',
			plugin_dir_url( ETHPRESS_FILE ) . 'public/dist/login-front.min.js',
			[ 'ethpress-login', 'ethpress-login-modal' ],
			'1.3.0',
			true
		);

		$ajax_url        = admin_url( 'admin-ajax.php' );
		$login_nonce     = wp_create_nonce( 'ethpress_log_in' );
		$get_nonce_nonce = wp_create_nonce( 'ethpress_get_message' );
		// wp_localize_script is for l10n, but, I mean, same thing.
		wp_add_inline_script(
			'ethpress-login',
			"
var ethpressLoginWP = {
	'ajaxUrl': '$ajax_url',
	'loginNonce': '$login_nonce',
	'getNonceNonce': '$get_nonce_nonce',
	'l10n': {
		'calltoaction': '" . __( 'Choose your login method', 'ethpress' ) . "',
		'nodetect': '" . __( 'Error: cannot detect crypto wallet', 'ethpress' ) . "',
		'permission': '" . __( 'Waiting for your permission', 'ethpress' ) . "',
		'fetching': '" . __( 'Fetching login phrase...', 'ethpress' ) . "',
		'awaiting': '" . __( 'Waiting for your signature', 'ethpress' ) . "',
		'verifying': '" . __( 'Verifying signature...', 'ethpress' ) . "',
		'loggedin': '" . __( 'Logged in', 'ethpress' ) . "',
		'aborted': '" . __( 'Login aborted', 'ethpress' ) . "',
		'heading': '" . __( 'Log In', 'ethpress' ) . "'
	}
};
			",
			'before'
		);
	}

	/**
	 * Attached to hook.
	 *
	 * @since 0.1.0
	 */
	public static function login_enqueue_scripts() {
		// Only enqueue the -front ones. They'll pull the other ones. This way it's easier to remove them.
		wp_enqueue_style( 'ethpress-login-front' );
		wp_enqueue_script( 'ethpress-login-front' );
	}

	/**
	 * Sends an error message if trying to log in with MetaMask while logged in.
	 *
	 * @since 0.1.0
	 * @deprecated
	 */
	public static function logged_in_user_error() {
		wp_send_json_error( esc_html__( 'Log out first', 'ethpress' ) );
	}

	/**
	 * Load translation files.
	 *
	 * @since 0.1.0
	 */
	public static function load_plugin_textdomain() {
		$path  = dirname( plugin_basename( ETHPRESS_FILE ) );
		$path .= '/languages';
		load_plugin_textdomain( 'ethpress', false, $path );
	}

	/**
	 * Is called from uninstall.php
	 *
	 * TODO: add a conditional ETHPRESS_REMOVE_USERS and remove all user accounts if true?
	 *
	 * @since 0.1.0
	 */
	public static function uninstall() {
		defined( 'WP_UNINSTALL_PLUGIN' ) || die;

		global $wpdb;

		$table = $wpdb->base_prefix . self::$tables['addresses'];

		// phpcs:ignore -- No cache, schema change, no prepare.
		$wpdb->query( "DROP TABLE IF EXISTS {$table}" );
		delete_option( 'ethpress' );
	}

	/**
	 * Creates database tables on plugin activation.
	 *
	 * @since 0.1.0
	 */
	public static function activate() {
		global $wpdb;

		$table      = $wpdb->base_prefix . self::$tables['addresses'];
		$db_version = '1.1';
		$api_url    = 'https://verify-eth-signature.herokuapp.com/ethpress';
		$opts       = [
			'db_version' => $db_version,
			'api_url'    => $api_url,
		];
		/**
		 * Ethpress' settings.
		 *
		 * @since 0.1.0
		 *
		 * Settings: db_version (internal), api_url: see Signature.php.
		 */
		add_site_option( 'ethpress', $opts );

		require_once ABSPATH . 'wp-admin/includes/upgrade.php';

		// No multisite stuff here because user table only exists once.
		$charset_collate = '';
		if ( $wpdb->has_cap( 'collation' ) ) {
			$charset_collate = $wpdb->get_charset_collate();
		}
		$sql = "
CREATE TABLE {$table} (
  id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
  name varchar(191) NOT NULL,
  user_id BIGINT UNSIGNED NOT NULL,
  coin varchar(15) DEFAULT NULL,
  date datetime DEFAULT '2000-01-01 00:00:00',
  modified datetime DEFAULT '2000-01-01 00:00:00',
  PRIMARY KEY  (id),
  UNIQUE KEY name (name),
  KEY user_id (user_id)
) $charset_collate;";
		\dbDelta( $sql );

		Upgrade::handle_upgrades();
	}
}
