'use strict';

/**
 * Handles login interaction with MetaMask.
 */
var ethpress = ( function( web3, ethereum, ethpressLoginWP ) {
	var _strings = ethpressLoginWP.strings,
		_that,
		_address,
		_err,
		_eventListeners = { status: [] },
		on,
		getLoginMessage,
		metamask,
		setStatus;

	// "Log in with {MetaMask,TrustWallet}".
	if ( ethereum.isTrust ) {
		document.addEventListener( 'DOMContentLoaded', function() {
			document.querySelectorAll( '.ethpress-metamask-login-button' )
				.forEach( function( element ) {
					element.innerText = _strings.trustWallet;
				});
		});
	}

	/**
	 * Adds listeners. Currently the only event is 'status'.
	 *
	 * @param string   what     What to listen for.
	 * @param function callback Calls this function.
	 *
	 * @since 0.3.0
	 */
	on = function( what, callback ) {
		if ( 'function' !== typeof callback ) {
			throw new Error( 'EthPress: Only functions work as listeners' );
		}
		if ( ! _eventListeners[what]) {
			_eventListeners[what] = [];
		}
		_eventListeners[what].push( callback );
	};

	/**
	 * Get nonce for current wallet, or register. Call window.ethereum.enable before calling this.
	 *
	 * Internal function.
	 *
	 * @param array[string] addresses Metamask addresses.
	 */
	getLoginMessage = function( addresses ) {
		var data = new URLSearchParams();

		// Using web3.eth.coinbase is unreliable.
		// https://github.com/MetaMask/metamask-extension/issues/6674
		_address = addresses[0];
		data.append( 'action', 'ethpress_get_message' );
		data.append( '_ajax_nonce', ethpressLoginWP.getNonceNonce );
		data.append( 'coinbase', _address );
		return fetch( ethpressLoginWP.ajaxUrl + '?' + data.toString() )
			.catch( function( e ) {
				setStatus( _strings.unknownError, false );
				throw e;
			});
	};

	/**
	 * Sets button text.
	 *
	 * @param string statusText Short text describing status.
	 * @param bool   disable    True if button should be disabled, false if enabled, and null if no change.
	 */
	setStatus = function( statusText, disable ) {
		if ( _eventListeners.status.length ) {
			_eventListeners.status.forEach( function( callback ) {
				callback({
					error: statusText instanceof Error,
					message: statusText,
					buttonDisabled: disable,
					type: 'metamask'
				});
			});
		}

		if ( _that instanceof Element ) {
			if ( true === disable ) {
				_that.disabled = true;
			} else if ( false === disable ) {
				_that.disabled = false;
			}

			_that.innerText = statusText;
		}
	};

	metamask = {

		/**
		 * Register and log in.
		 */
		connect: function() {

			// Status texts will be displayed on the button element itself.
			_that = this;

			if ( ! web3 || ! web3.personal || ! ethereum ) {
				setStatus( _strings.notInstalled, true );
				return;
			}

			// Disable button during login procedures.
			setStatus( _strings.waitPermission, true );

			ethereum.enable()
			.then( function( addresses ) {
				setStatus( _strings.fetchingDocument );
				return addresses;
			})
			.then( getLoginMessage )
			.then( function( res ) {
				if ( res.ok ) {
					return res.json();
				}
				throw new Error( res.statusText );
			})
			.then( function( result ) {
				if ( ! result.success ) {
					throw new Error( result.data );
				}

				setStatus( _strings.waitSignature );

				// Message was signed, now verify it.
				web3.personal.sign( web3.fromUtf8( result.data ), _address, metamask.verify );
			})
			.catch( function( err ) {

				// Cancelled at "Connect" screen.
				if ( null != err.code && -32603 === err.code ) {
					err.message = _strings.aborted;
				}

				// Enable button on error so user can retry.
				setStatus( err.message, false );
			});
		},

		verify: function( err, signedMessage ) {
			var data = new FormData();

			if ( null != err ) {
				setStatus( _strings.aborted, false );
				return;
			}

			if ( 'string' !== typeof signedMessage ) {
				setStatus( _strings.unknownError, false );
				return;
			}

			data.append( 'signature', signedMessage );
			data.append( '_ajax_nonce', ethpressLoginWP.loginNonce );
			data.append( 'action', 'ethpress_log_in' );
			data.append( 'coinbase', _address );
			data.append( 'redirect_to', new URLSearchParams( window.location.search ).get( 'redirect_to' ) || '' );

			setStatus( _strings.verifySignature );

			fetch( ethpressLoginWP.ajaxUrl, {
				method: 'POST',
				body: data,
				credentials: 'same-origin'
			})
			.then( function( res ) {
				if ( res.ok ) {
					return res.json();
				}
				throw new Error( res.statusText );
			})
			.then( function( result ) {
				if ( result.success ) {
					return result.data;
				}
				_err = _strings.unknownError;
				if ( result.data && result.data.message ) {
					_err = result.data.message;
				}
				throw new Error( _err );
			})
			.then( function( data ) {

				// Succesful login.
				setStatus( data.message );

				document.location.href = data.redirect;
			})
			.catch( function( reason ) {

				// Failed login. Enable button again.
				setStatus( reason, false );
			});
		}
	};

	return {
		metamask: metamask,
		on: on
	};
}( window.web3, window.ethereum, window.ethpressLoginWP ) );
